@@ -70,14 +70,34 @@ class PassManagerBuilder(llvm.Wrapper):
     def size_level(self, sizelevel):
         self._ptr.SizeLevel = sizelevel
 
-    @property
-    def vectorize(self):
-        return self._ptr.Vectorize
+    if llvm.version >= (3, 3):
+        @property
+        def bbvectorize(self):
+            return self._ptr.BBVectorize
+
+        @bbvectorize.setter
+        def bbvectorize(self, enable):
+            self._ptr.BBVectorize = enable
+
+        vectorize = bbvectorize
+
+        @property
+        def slpvectorize(self):
+            return self._ptr.SLPVectorize
+
+        @slpvectorize.setter
+        def slpvectorize(self, enable):
+            self._ptr.SLPVectorize = enable
+
+    else:
+        @property
+        def vectorize(self):
+            return self._ptr.Vectorize
+
+        @vectorize.setter
+        def vectorize(self, enable):
+            self._ptr.Vectorize = enable
 
-    @vectorize.setter
-    def vectorize(self, enable):
-        self._ptr.Vectorize = enable
-    
 
     @property
     def loop_vectorize(self):
@@ -109,13 +129,14 @@ class PassManagerBuilder(llvm.Wrapper):
     def disable_unroll_loops(self, disable):
         self._ptr.DisableUnrollLoops = disable
 
-    @property
-    def disable_simplify_lib_calls(self):
-        return self._ptr.DisableSimplifyLibCalls
+    if llvm.version <= (3, 3):
+        @property
+        def disable_simplify_lib_calls(self):
+            return self._ptr.DisableSimplifyLibCalls
 
-    @disable_simplify_lib_calls.setter
-    def disable_simplify_lib_calls(self, disable):
-        self._ptr.DisableSimplifyLibCalls = disable
+        @disable_simplify_lib_calls.setter
+        def disable_simplify_lib_calls(self, disable):
+            self._ptr.DisableSimplifyLibCalls = disable
 
     def use_inliner_with_threshold(self, threshold):
         self._ptr.Inliner = api.llvm.createFunctionInliningPass(threshold)
@@ -200,7 +221,7 @@ class Pass(llvm.Wrapper):
         try:
             return self.__name
         except AttributeError:
-            return 
+            return
 
     @property
     def description(self):
@@ -299,13 +320,15 @@ class TargetTransformInfo(Pass):
 # Helpers
 #===----------------------------------------------------------------------===
 
-def build_pass_managers(tm, opt=2, loop_vectorize=False, vectorize=False,
-                        inline_threshold=2000, pm=True, fpm=True, mod=None):
+def build_pass_managers(tm, opt=2, size=0, loop_vectorize=False,
+                        slp_vectorize=False, vectorize=False,
+                        inline_threshold=None, pm=True, fpm=True, mod=None):
     '''
         tm --- The TargetMachine for which the passes are optimizing for.
         The TargetMachine must stay alive until the pass managers
         are removed.
         opt --- [0-3] Optimization level. Default to 2.
+        size --- [0-2] Optimize for size. Default to 0.
         loop_vectorize --- [boolean] Whether to use loop-vectorizer.
         vectorize --- [boolean] Whether to use basic-block vectorizer.
         inline_threshold --- [int] Threshold for the inliner.
@@ -314,6 +337,18 @@ def build_pass_managers(tm, opt=2, loop_vectorize=False, vectorize=False,
         fpm --- [boolean] Whether to build a function-level pass-manager.
         mod --- [Module] The module object for the FunctionPassManager.
         '''
+    if inline_threshold is None:
+        if 0 < opt < 3:
+            inline_threshold = 225
+
+        if size == 1:
+            inline_threshold = 75
+        elif size == 2:
+            inline_threshold = 25
+
+        if opt >= 3:
+            inline_threshold = 275
+
     if pm:
         pm = PassManager.new()
     if fpm:
@@ -326,22 +361,27 @@ def build_pass_managers(tm, opt=2, loop_vectorize=False, vectorize=False,
     pmb.opt_level = opt
     pmb.vectorize = vectorize
     pmb.loop_vectorize = loop_vectorize
+    if llvm.version >= (3, 3):
+        pmb.slp_vectorize = slp_vectorize
     if inline_threshold:
         pmb.use_inliner_with_threshold(inline_threshold)
     if pm:
         pm.add(tm.target_data.clone())
         pm.add(TargetLibraryInfo.new(tm.triple))
-        if llvm.version >= (3, 2):
+        if llvm.version <= (3, 2):
             pm.add(TargetTransformInfo.new(tm))
+        else:
+            tm.add_analysis_passes(pm)
         pmb.populate(pm)
 
     if fpm:
-        fpm.add(tm.target_data)
+        fpm.add(tm.target_data.clone())
         fpm.add(TargetLibraryInfo.new(tm.triple))
-        if llvm.version >= (3, 2):
+        if llvm.version <= (3, 2):
             fpm.add(TargetTransformInfo.new(tm))
+        else:
+            tm.add_analysis_passes(fpm)
         pmb.populate(fpm)
-        fpm.initialize()
 
     from collections import namedtuple
     return namedtuple('passmanagers', ['pm', 'fpm'])(pm=pm, fpm=fpm)
